module BittrexEnterprise
  class Orders
    include ApiHelpers

  # ==============================================================================================
  # AUTH REQUIRED ================================================================================
  # ==============================================================================================

    # ----------------------------------------- OPEN ---------------------------------------------
    # List open deposits. Results are sorted in inverse order of UpdatedAt, and are limited to
    # the first 1000.
    #
    # ------ PARAMS ------ PARAMS MUST BE PASSED AS KEY VALUE PAIRS
    # marketSymbol - string - optional - filter by market
    # --------------------------------------------------------------------------------------------
    def self.open(params={})
      get_signed 'orders/open', params
    end


    # ---------------------------------------- CLOSED --------------------------------------------
    # List closed orders.
    # StartDate and EndDate filters apply to the ClosedAt field. Pagination and the sort order
    # of the results are in inverse order of the ClosedAt field.
    #
    # ------ PARAMS ------ PARAMS MUST BE PASSED AS KEY VALUE PAIRS
    # marketSymbol - string - optional - filter by market
    #
    # nextPageToken - string - The unique identifier of the item that the resulting query result
    #         should start after, in the sort order of the given endpoint. Used for traversing
    #         a paginated set in the forward direction. (May only be specified if PreviousPageToken
    #         is not specified.)
    #
    # previousPageToken - string - The unique identifier of the item that the resulting query
    #         result should end before, in the sort order of the given endpoint. Used for traversing
    #         a paginated set in the reverse direction. (May only be specified if NextPageToken is
    #         not specified.)
    #
    # pageSize - integer - maximum number of items to retrieve -- default 100, minimum 1, maximum 200
    #
    # startDate - string(date-time) - Filters out results before this timestamp. In ISO 8601 format
    #         (e.g., "2019-01-02T16:23:45Z"). Precision beyond one second is not supported. Use
    #         pagination parameters for more precise filtering.
    #
    # endDate - string(date-time) - Filters out result after this timestamp. Uses the same format
    #         as StartDate. Either, both, or neither of StartDate and EndDate can be set. The only
    #         constraint on the pair is that, if both are set, then EndDate cannot be before StartDate.
    # --------------------------------------------------------------------------------------------
    def self.closed(params={})
      get_signed 'orders/closed', params
    end


    # --------------------------------------- RETRIEVE -------------------------------------------
    # Retrieve information on a specific order.
    #
    # ------ PARAMS ------
    # order_id - string - required - (guid-formatted string) - ID of the order to retrieve
    # --------------------------------------------------------------------------------------------
    def self.retrieve(order_id)
      get_signed 'orders/{orderId}', orderId: order_id
    end

    # ---------------------------------------- CANCEL --------------------------------------------
    # Cancel an order.
    #
    # ------ PARAMS ------
    # order_id - string - required - (guid-formatted string) - ID of the order to retrieve
    # --------------------------------------------------------------------------------------------
    def self.cancel(order_id)
      delete_signed 'orders/{orderId}', orderId: order_id
    end


    # ---------------------------------------- CREATE --------------------------------------------
    # Create a new order.
    #
    # ------ PARAMS ------
    # NewOrder - object - required - information specifying the order to create
    #         {
    #           "marketSymbol": "string",
    #           "direction": "string",
    #           "type": "string",
    #           "quantity": "number (double)",
    #           "ceiling": "number (double)",
    #           "limit": "number (double)",
    #           "timeInForce": "string",
    #           "expiresAt": "string (date-time)",
    #           "clientOrderId": "string (uuid)"
    #         }
    #   *** REQUIRED - marketSymbol, direction, type ***
    # --------------------------------------------------------------------------------------------
    def self.create(new_order)
      post_signed 'orders', new_order
    end

  end
end
