module BittrexEnterprise
  class Withdrawals
    include ApiHelpers

  # ==============================================================================================
  # AUTH REQUIRED ================================================================================
  # ==============================================================================================

    # ----------------------------------------- OPEN ---------------------------------------------
    # List open withdrawals. Results are sorted in inverse order of the CreatedAt field, and are
    # limited to the first 1000.
    #
    # ------ PARAMS ------ PARAMS MUST BE PASSED AS KEY VALUE PAIRS
    # status - string enum: [REQUESTED, AUTHORIZED, PENDING, ERROR_INVALID_ADDRESS] - optional -
    #                                                      filter by an open withdrawal status
    #
    # currencySymbol - string - optional - filter by currency
    #
    # sub_account_id: string - optional - ID of the subaccount to use for this call
    # --------------------------------------------------------------------------------------------
    def self.open(params={})
      get_signed 'withdrawals/open', params
    end


    # ---------------------------------------- CLOSED --------------------------------------------
    # List closed withdrawals. StartDate and EndDate filters apply to the CompletedAt field.
    # Pagination and the sort order of the results are in inverse order of the CompletedAt field.
    #
    # ------ PARAMS ------ PARAMS MUST BE PASSED AS KEY VALUE PAIRS
    # status - string enum: [COMPLETED, CANCELLED] - optional - filter by an open withdrawal status
    #
    # currencySymbol - string - optional - filter by currency
    #
    # nextPageToken - string - The unique identifier of the item that the resulting query result
    #         should start after, in the sort order of the given endpoint. Used for traversing
    #         a paginated set in the forward direction. (May only be specified if PreviousPageToken
    #         is not specified.)
    #
    # previousPageToken - string - The unique identifier of the item that the resulting query
    #         result should end before, in the sort order of the given endpoint. Used for traversing
    #         a paginated set in the reverse direction. (May only be specified if NextPageToken is
    #         not specified.)
    #
    # pageSize - integer - maximum number of items to retrieve -- default 100, minimum 1, maximum 200
    #
    # startDate - string(date-time) - Filters out results before this timestamp. In ISO 8601 format
    #         (e.g., "2019-01-02T16:23:45Z"). Precision beyond one second is not supported. Use
    #         pagination parameters for more precise filtering.
    #
    # endDate - string(date-time) - Filters out result after this timestamp. Uses the same format
    #         as StartDate. Either, both, or neither of StartDate and EndDate can be set. The only
    #         constraint on the pair is that, if both are set, then EndDate cannot be before StartDate.
    #
    # sub_account_id: string - optional - ID of the subaccount to use for this call
    # --------------------------------------------------------------------------------------------
    def self.closed(params={})
      get_signed 'withdrawals/closed', params
    end


    # --------------------------------------- BY TX ID -------------------------------------------
    # Retrieves all withdrawals for this account with the given TxId
    #
    # ------ PARAMS ------
    # tx_id - string - required - the transaction id to lookup
    #
    # sub_account_id: string - optional - ID of the subaccount to use for this call
    # --------------------------------------------------------------------------------------------
    def self.by_tx_id(tx_id, sub_account_id='')
      get_signed 'withdrawals/ByTxId/{txId}', txId: tx_id, sub_account_id: sub_account_id
    end


    # --------------------------------------- RETRIEVE -------------------------------------------
    # Retrieve information on a specified withdrawal.
    #
    # ------ PARAMS ------
    # withdrawal_id - string - required - (guid-formatted string) - ID of withdrawal to retrieve
    #
    # sub_account_id: string - optional - ID of the subaccount to use for this call
    # --------------------------------------------------------------------------------------------
    def self.retrieve(withdrawal_id, sub_account_id='')
      get_signed 'withdrawals/{withdrawalId}', withdrawalId: withdrawal_id, sub_account_id: sub_account_id
    end


    # ---------------------------------------- CANCEL --------------------------------------------
    # Cancel an order.
    #
    # ------ PARAMS ------
    # withdrawal_id - string - required - (guid-formatted string) - ID of withdrawal to cancel
    #
    # sub_account_id: string - optional - ID of the subaccount to use for this call
    # --------------------------------------------------------------------------------------------
    def self.cancel(withdrawal_id, sub_account_id='')
      delete_signed 'withdrawals/{withdrawalId}', withdrawalId: withdrawal_id, sub_account_id: sub_account_id
    end


    # ---------------------------------------- CREATE --------------------------------------------
    # Create a new withdrawal.
    #
    # ------ PARAMS ------
    # NewWithdrawal - object - required - information specifying the withdrawal to create
    #         {
    #           "currencySymbol": "string",
    #           "quantity": "number (double)",
    #           "cryptoAddress": "string",
    #           "cryptoAddressTag": "string",
    #           "sub_account_id": "string"
    #         }
    #   *** REQUIRED - currencySymbol, quantity, cryptoAddress  ***
    #
    # sub_account_id: string - optional - ID of the subaccount to use for this call
    # --------------------------------------------------------------------------------------------
    def self.create(new_withdrawal={})
      post_signed 'withdrawals', new_withdrawal
    end

  end
end
